# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class Task(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "simulator",
            "status_alias",
            "project",
            "task_id",
            "is_terminated",
            "status",
        }

        class properties:
            task_id = schemas.StrSchema

            @staticmethod
            def status() -> typing.Type['TaskStatusCode']:
                return TaskStatusCode

            status_alias = schemas.StrSchema
            simulator = schemas.StrSchema
            project = schemas.StrSchema
            is_terminated = schemas.BoolSchema

            class storage_path(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'storage_path':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class storage_input_path(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'storage_input_path':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class storage_output_path(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'storage_output_path':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class container_image(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'container_image':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class create_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'create_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_submit_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'input_submit_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class start_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'start_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class computation_start_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'computation_start_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class computation_end_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'computation_end_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class end_time(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'end_time':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class estimated_computation_cost(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.NumberSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'estimated_computation_cost':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class storage_size(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.IntSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'storage_size':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class metrics(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            TaskMetrics,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'metrics':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class executer(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            Executer,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'executer':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class machine_group_name(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'machine_group_name':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class machine_group_id(
                    schemas.UUIDBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'uuid'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'machine_group_id':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class error_detail(
                    schemas.ComposedSchema,):

                class MetaOapg:
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'error_detail':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            class input_resources(schemas.ListSchema):

                class MetaOapg:
                    items = schemas.StrSchema

                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple[typing.Union[
                        MetaOapg.items,
                        str,
                    ]], typing.List[typing.Union[
                        MetaOapg.items,
                        str,
                    ]]],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                ) -> 'input_resources':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )

                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)

            stream_zip = schemas.BoolSchema
            __annotations__ = {
                "task_id": task_id,
                "status": status,
                "status_alias": status_alias,
                "simulator": simulator,
                "project": project,
                "is_terminated": is_terminated,
                "storage_path": storage_path,
                "storage_input_path": storage_input_path,
                "storage_output_path": storage_output_path,
                "container_image": container_image,
                "create_time": create_time,
                "input_submit_time": input_submit_time,
                "start_time": start_time,
                "computation_start_time": computation_start_time,
                "computation_end_time": computation_end_time,
                "end_time": end_time,
                "estimated_computation_cost": estimated_computation_cost,
                "storage_size": storage_size,
                "metrics": metrics,
                "executer": executer,
                "machine_group_name": machine_group_name,
                "machine_group_id": machine_group_id,
                "error_detail": error_detail,
                "input_resources": input_resources,
                "stream_zip": stream_zip,
            }

    simulator: MetaOapg.properties.simulator
    status_alias: MetaOapg.properties.status_alias
    project: MetaOapg.properties.project
    task_id: MetaOapg.properties.task_id
    is_terminated: MetaOapg.properties.is_terminated
    status: 'TaskStatusCode'

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["task_id"]
    ) -> MetaOapg.properties.task_id:
        ...

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["status"]) -> 'TaskStatusCode':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["status_alias"]
    ) -> MetaOapg.properties.status_alias:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["simulator"]
    ) -> MetaOapg.properties.simulator:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["project"]
    ) -> MetaOapg.properties.project:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["is_terminated"]
    ) -> MetaOapg.properties.is_terminated:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["storage_path"]
    ) -> MetaOapg.properties.storage_path:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["storage_input_path"]
    ) -> MetaOapg.properties.storage_input_path:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["storage_output_path"]
    ) -> MetaOapg.properties.storage_output_path:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["container_image"]
    ) -> MetaOapg.properties.container_image:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["create_time"]
    ) -> MetaOapg.properties.create_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_submit_time"]
    ) -> MetaOapg.properties.input_submit_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["start_time"]
    ) -> MetaOapg.properties.start_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["computation_start_time"]
    ) -> MetaOapg.properties.computation_start_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["computation_end_time"]
    ) -> MetaOapg.properties.computation_end_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["end_time"]
    ) -> MetaOapg.properties.end_time:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["estimated_computation_cost"]
    ) -> MetaOapg.properties.estimated_computation_cost:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["storage_size"]
    ) -> MetaOapg.properties.storage_size:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["metrics"]
    ) -> MetaOapg.properties.metrics:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["executer"]
    ) -> MetaOapg.properties.executer:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["machine_group_name"]
    ) -> MetaOapg.properties.machine_group_name:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["machine_group_id"]
    ) -> MetaOapg.properties.machine_group_id:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["error_detail"]
    ) -> MetaOapg.properties.error_detail:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["input_resources"]
    ) -> MetaOapg.properties.input_resources:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["stream_zip"]
    ) -> MetaOapg.properties.stream_zip:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "task_id",
        "status",
        "status_alias",
        "simulator",
        "project",
        "is_terminated",
        "storage_path",
        "storage_input_path",
        "storage_output_path",
        "container_image",
        "create_time",
        "input_submit_time",
        "start_time",
        "computation_start_time",
        "computation_end_time",
        "end_time",
        "estimated_computation_cost",
        "storage_size",
        "metrics",
        "executer",
        "machine_group_name",
        "machine_group_id",
        "error_detail",
        "input_resources",
        "stream_zip",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["task_id"]
    ) -> MetaOapg.properties.task_id:
        ...

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["status"]) -> 'TaskStatusCode':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["status_alias"]
    ) -> MetaOapg.properties.status_alias:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["simulator"]
    ) -> MetaOapg.properties.simulator:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["project"]
    ) -> MetaOapg.properties.project:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["is_terminated"]
    ) -> MetaOapg.properties.is_terminated:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["storage_path"]
    ) -> typing.Union[MetaOapg.properties.storage_path, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["storage_input_path"]
    ) -> typing.Union[MetaOapg.properties.storage_input_path, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["storage_output_path"]
    ) -> typing.Union[MetaOapg.properties.storage_output_path, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["container_image"]
    ) -> typing.Union[MetaOapg.properties.container_image, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["create_time"]
    ) -> typing.Union[MetaOapg.properties.create_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_submit_time"]
    ) -> typing.Union[MetaOapg.properties.input_submit_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["start_time"]
    ) -> typing.Union[MetaOapg.properties.start_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["computation_start_time"]
    ) -> typing.Union[MetaOapg.properties.computation_start_time,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["computation_end_time"]
    ) -> typing.Union[MetaOapg.properties.computation_end_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["end_time"]
    ) -> typing.Union[MetaOapg.properties.end_time, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["estimated_computation_cost"]
    ) -> typing.Union[MetaOapg.properties.estimated_computation_cost,
                      schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["storage_size"]
    ) -> typing.Union[MetaOapg.properties.storage_size, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["metrics"]
    ) -> typing.Union[MetaOapg.properties.metrics, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["executer"]
    ) -> typing.Union[MetaOapg.properties.executer, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["machine_group_name"]
    ) -> typing.Union[MetaOapg.properties.machine_group_name, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["machine_group_id"]
    ) -> typing.Union[MetaOapg.properties.machine_group_id, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["error_detail"]
    ) -> typing.Union[MetaOapg.properties.error_detail, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["input_resources"]
    ) -> typing.Union[MetaOapg.properties.input_resources, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["stream_zip"]
    ) -> typing.Union[MetaOapg.properties.stream_zip, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "task_id",
        "status",
        "status_alias",
        "simulator",
        "project",
        "is_terminated",
        "storage_path",
        "storage_input_path",
        "storage_output_path",
        "container_image",
        "create_time",
        "input_submit_time",
        "start_time",
        "computation_start_time",
        "computation_end_time",
        "end_time",
        "estimated_computation_cost",
        "storage_size",
        "metrics",
        "executer",
        "machine_group_name",
        "machine_group_id",
        "error_detail",
        "input_resources",
        "stream_zip",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        simulator: typing.Union[
            MetaOapg.properties.simulator,
            str,
        ],
        status_alias: typing.Union[
            MetaOapg.properties.status_alias,
            str,
        ],
        project: typing.Union[
            MetaOapg.properties.project,
            str,
        ],
        task_id: typing.Union[
            MetaOapg.properties.task_id,
            str,
        ],
        is_terminated: typing.Union[
            MetaOapg.properties.is_terminated,
            bool,
        ],
        status: 'TaskStatusCode',
        storage_path: typing.Union[MetaOapg.properties.storage_path, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        storage_input_path: typing.Union[MetaOapg.properties.storage_input_path,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        storage_output_path: typing.Union[
            MetaOapg.properties.storage_output_path, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        container_image: typing.Union[MetaOapg.properties.container_image, dict,
                                      frozendict.frozendict, str, date,
                                      datetime, uuid.UUID, int, float,
                                      decimal.Decimal, bool, None, list, tuple,
                                      bytes, io.FileIO, io.BufferedReader,
                                      schemas.Unset] = schemas.unset,
        create_time: typing.Union[MetaOapg.properties.create_time, dict,
                                  frozendict.frozendict, str, date, datetime,
                                  uuid.UUID, int, float, decimal.Decimal, bool,
                                  None, list, tuple, bytes, io.FileIO,
                                  io.BufferedReader,
                                  schemas.Unset] = schemas.unset,
        input_submit_time: typing.Union[MetaOapg.properties.input_submit_time,
                                        dict, frozendict.frozendict, str, date,
                                        datetime, uuid.UUID, int, float,
                                        decimal.Decimal, bool, None, list,
                                        tuple, bytes, io.FileIO,
                                        io.BufferedReader,
                                        schemas.Unset] = schemas.unset,
        start_time: typing.Union[MetaOapg.properties.start_time, dict,
                                 frozendict.frozendict, str, date, datetime,
                                 uuid.UUID, int, float, decimal.Decimal, bool,
                                 None, list, tuple, bytes, io.FileIO,
                                 io.BufferedReader,
                                 schemas.Unset] = schemas.unset,
        computation_start_time: typing.Union[
            MetaOapg.properties.computation_start_time, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        computation_end_time: typing.Union[
            MetaOapg.properties.computation_end_time, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        end_time: typing.Union[MetaOapg.properties.end_time, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, bool,
                               None, list, tuple, bytes, io.FileIO,
                               io.BufferedReader,
                               schemas.Unset] = schemas.unset,
        estimated_computation_cost: typing.Union[
            MetaOapg.properties.estimated_computation_cost, dict,
            frozendict.frozendict, str, date, datetime, uuid.UUID, int, float,
            decimal.Decimal, bool, None, list, tuple, bytes, io.FileIO,
            io.BufferedReader, schemas.Unset] = schemas.unset,
        storage_size: typing.Union[MetaOapg.properties.storage_size, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        metrics: typing.Union[MetaOapg.properties.metrics, dict,
                              frozendict.frozendict, str, date, datetime,
                              uuid.UUID, int, float, decimal.Decimal, bool,
                              None, list, tuple, bytes, io.FileIO,
                              io.BufferedReader, schemas.Unset] = schemas.unset,
        executer: typing.Union[MetaOapg.properties.executer, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, bool,
                               None, list, tuple, bytes, io.FileIO,
                               io.BufferedReader,
                               schemas.Unset] = schemas.unset,
        machine_group_name: typing.Union[MetaOapg.properties.machine_group_name,
                                         dict, frozendict.frozendict, str, date,
                                         datetime, uuid.UUID, int, float,
                                         decimal.Decimal, bool, None, list,
                                         tuple, bytes, io.FileIO,
                                         io.BufferedReader,
                                         schemas.Unset] = schemas.unset,
        machine_group_id: typing.Union[MetaOapg.properties.machine_group_id,
                                       dict, frozendict.frozendict, str, date,
                                       datetime, uuid.UUID, int, float,
                                       decimal.Decimal, bool, None, list, tuple,
                                       bytes, io.FileIO, io.BufferedReader,
                                       schemas.Unset] = schemas.unset,
        error_detail: typing.Union[MetaOapg.properties.error_detail, dict,
                                   frozendict.frozendict, str, date, datetime,
                                   uuid.UUID, int, float, decimal.Decimal, bool,
                                   None, list, tuple, bytes, io.FileIO,
                                   io.BufferedReader,
                                   schemas.Unset] = schemas.unset,
        input_resources: typing.Union[MetaOapg.properties.input_resources, list,
                                      tuple, schemas.Unset] = schemas.unset,
        stream_zip: typing.Union[MetaOapg.properties.stream_zip, bool,
                                 schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'Task':
        return super().__new__(
            cls,
            *_args,
            simulator=simulator,
            status_alias=status_alias,
            project=project,
            task_id=task_id,
            is_terminated=is_terminated,
            status=status,
            storage_path=storage_path,
            storage_input_path=storage_input_path,
            storage_output_path=storage_output_path,
            container_image=container_image,
            create_time=create_time,
            input_submit_time=input_submit_time,
            start_time=start_time,
            computation_start_time=computation_start_time,
            computation_end_time=computation_end_time,
            end_time=end_time,
            estimated_computation_cost=estimated_computation_cost,
            storage_size=storage_size,
            metrics=metrics,
            executer=executer,
            machine_group_name=machine_group_name,
            machine_group_id=machine_group_id,
            error_detail=error_detail,
            input_resources=input_resources,
            stream_zip=stream_zip,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.executer import Executer
from inductiva.client.model.task_metrics import TaskMetrics
from inductiva.client.model.task_status_code import TaskStatusCode



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class DynamicDiskResizeConfig(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "free_space_threshold_gb",
            "size_increment_gb",
            "max_disk_size_gb",
        }

        class properties:
            free_space_threshold_gb = schemas.IntSchema
            size_increment_gb = schemas.IntSchema
            max_disk_size_gb = schemas.IntSchema
            __annotations__ = {
                "free_space_threshold_gb": free_space_threshold_gb,
                "size_increment_gb": size_increment_gb,
                "max_disk_size_gb": max_disk_size_gb,
            }

    free_space_threshold_gb: MetaOapg.properties.free_space_threshold_gb
    size_increment_gb: MetaOapg.properties.size_increment_gb
    max_disk_size_gb: MetaOapg.properties.max_disk_size_gb

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["free_space_threshold_gb"]
    ) -> MetaOapg.properties.free_space_threshold_gb:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["size_increment_gb"]
    ) -> MetaOapg.properties.size_increment_gb:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["max_disk_size_gb"]
    ) -> MetaOapg.properties.max_disk_size_gb:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "free_space_threshold_gb",
        "size_increment_gb",
        "max_disk_size_gb",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["free_space_threshold_gb"]
    ) -> MetaOapg.properties.free_space_threshold_gb:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["size_increment_gb"]
    ) -> MetaOapg.properties.size_increment_gb:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["max_disk_size_gb"]
    ) -> MetaOapg.properties.max_disk_size_gb:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "free_space_threshold_gb",
        "size_increment_gb",
        "max_disk_size_gb",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        free_space_threshold_gb: typing.Union[
            MetaOapg.properties.free_space_threshold_gb,
            decimal.Decimal,
            int,
        ],
        size_increment_gb: typing.Union[
            MetaOapg.properties.size_increment_gb,
            decimal.Decimal,
            int,
        ],
        max_disk_size_gb: typing.Union[
            MetaOapg.properties.max_disk_size_gb,
            decimal.Decimal,
            int,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'DynamicDiskResizeConfig':
        return super().__new__(
            cls,
            *_args,
            free_space_threshold_gb=free_space_threshold_gb,
            size_increment_gb=size_increment_gb,
            max_disk_size_gb=max_disk_size_gb,
            _configuration=_configuration,
            **kwargs,
        )



"""This module contains utility functions for handling authentication. """

import re

from inductiva import constants


def is_valid_token(token):
    """Validate token characters."""
    pattern = re.compile(r"^[A-Za-z0-9._-]+$")
    if not pattern.match(token):
        return False
    return True


def get_stored_api_key(path=None):
    """Returns the stored API key from the file system."""
    key_path = path or constants.API_KEY_FILE_PATH
    if not key_path.exists():
        return ""
    with open(key_path, "r", encoding="utf-8") as f:
        return f.read()


def set_stored_api_key(api_key, path=None):
    """Saves the API key to the file system."""
    key_path = path or constants.API_KEY_FILE_PATH
    with open(key_path, "w", encoding="utf-8") as f:
        f.write(api_key)


def remove_stored_api_key(path=None):
    """Removes the stored API key from the file system."""
    key_path = path or constants.API_KEY_FILE_PATH
    if key_path.exists():
        key_path.unlink()



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class TaskOperation(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "start_timestamp",
            "name",
            "alias",
            "attributes",
            "status",
        }

        class properties:

            @staticmethod
            def name() -> typing.Type['TaskOperationName']:
                return TaskOperationName

            alias = schemas.StrSchema
            start_timestamp = schemas.DateTimeSchema

            @staticmethod
            def status() -> typing.Type['OperationStatus']:
                return OperationStatus

            attributes = schemas.DictSchema

            class end_timestamp(
                    schemas.DateTimeBase,
                    schemas.ComposedSchema,
            ):

                class MetaOapg:
                    format = 'date-time'
                    any_of_0 = schemas.StrSchema
                    any_of_1 = schemas.NoneSchema

                    @classmethod
                    @functools.lru_cache()
                    def any_of(cls):
                        # we need this here to make our import statements work
                        # we must store _composed_schemas in here so the code is only run
                        # when we invoke this method. If we kept this at the class
                        # level we would get an error because the class level
                        # code would be run when this module is imported, and these composed
                        # classes don't exist yet because their module has not finished
                        # loading
                        return [
                            cls.any_of_0,
                            cls.any_of_1,
                        ]

                def __new__(
                    cls,
                    *_args: typing.Union[
                        dict,
                        frozendict.frozendict,
                        str,
                        date,
                        datetime,
                        uuid.UUID,
                        int,
                        float,
                        decimal.Decimal,
                        bool,
                        None,
                        list,
                        tuple,
                        bytes,
                        io.FileIO,
                        io.BufferedReader,
                    ],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                    **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                                           frozendict.frozendict, str, date,
                                           datetime, uuid.UUID, int, float,
                                           decimal.Decimal, None, list, tuple,
                                           bytes],
                ) -> 'end_timestamp':
                    return super().__new__(
                        cls,
                        *_args,
                        _configuration=_configuration,
                        **kwargs,
                    )

            __annotations__ = {
                "name": name,
                "alias": alias,
                "start_timestamp": start_timestamp,
                "status": status,
                "attributes": attributes,
                "end_timestamp": end_timestamp,
            }

    start_timestamp: MetaOapg.properties.start_timestamp
    name: 'TaskOperationName'
    alias: MetaOapg.properties.alias
    attributes: MetaOapg.properties.attributes
    status: 'OperationStatus'

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["name"]) -> 'TaskOperationName':
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["alias"]
    ) -> MetaOapg.properties.alias:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["start_timestamp"]
    ) -> MetaOapg.properties.start_timestamp:
        ...

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["status"]) -> 'OperationStatus':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["attributes"]
    ) -> MetaOapg.properties.attributes:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["end_timestamp"]
    ) -> MetaOapg.properties.end_timestamp:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "name",
        "alias",
        "start_timestamp",
        "status",
        "attributes",
        "end_timestamp",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["name"]) -> 'TaskOperationName':
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["alias"]
    ) -> MetaOapg.properties.alias:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["start_timestamp"]
    ) -> MetaOapg.properties.start_timestamp:
        ...

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["status"]) -> 'OperationStatus':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["attributes"]
    ) -> MetaOapg.properties.attributes:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["end_timestamp"]
    ) -> typing.Union[MetaOapg.properties.end_timestamp, schemas.Unset]:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "name",
        "alias",
        "start_timestamp",
        "status",
        "attributes",
        "end_timestamp",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        start_timestamp: typing.Union[
            MetaOapg.properties.start_timestamp,
            str,
            datetime,
        ],
        name: 'TaskOperationName',
        alias: typing.Union[
            MetaOapg.properties.alias,
            str,
        ],
        attributes: typing.Union[
            MetaOapg.properties.attributes,
            dict,
            frozendict.frozendict,
        ],
        status: 'OperationStatus',
        end_timestamp: typing.Union[MetaOapg.properties.end_timestamp, dict,
                                    frozendict.frozendict, str, date, datetime,
                                    uuid.UUID, int, float, decimal.Decimal,
                                    bool, None, list, tuple, bytes, io.FileIO,
                                    io.BufferedReader,
                                    schemas.Unset] = schemas.unset,
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'TaskOperation':
        return super().__new__(
            cls,
            *_args,
            start_timestamp=start_timestamp,
            name=name,
            alias=alias,
            attributes=attributes,
            status=status,
            end_timestamp=end_timestamp,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.operation_status import OperationStatus
from inductiva.client.model.task_operation_name import TaskOperationName



import typing_extensions

from inductiva.client.paths import PathValues
from inductiva.client.apis.paths.tasks_auth import TasksAuth
from inductiva.client.apis.paths.tasks_submit import TasksSubmit
from inductiva.client.apis.paths.tasks_task_id_input_upload_url import TasksTaskIdInputUploadUrl
from inductiva.client.apis.paths.tasks_task_id_input_uploaded import TasksTaskIdInputUploaded
from inductiva.client.apis.paths.tasks_task_id import TasksTaskId
from inductiva.client.apis.paths.tasks import Tasks
from inductiva.client.apis.paths.tasks_task_id_status import TasksTaskIdStatus
from inductiva.client.apis.paths.tasks_task_id_output_list import TasksTaskIdOutputList
from inductiva.client.apis.paths.tasks_task_id_download_input_url import TasksTaskIdDownloadInputUrl
from inductiva.client.apis.paths.tasks_task_id_download_output_url import TasksTaskIdDownloadOutputUrl
from inductiva.client.apis.paths.tasks_task_id_resubmit import TasksTaskIdResubmit
from inductiva.client.apis.paths.tasks_task_id_kill import TasksTaskIdKill
from inductiva.client.apis.paths.tasks_task_id_disable_logs import TasksTaskIdDisableLogs
from inductiva.client.apis.paths.tasks_task_id_files import TasksTaskIdFiles
from inductiva.client.apis.paths.tasks_task_id_register import TasksTaskIdRegister
from inductiva.client.apis.paths.tasks_task_id_offer import TasksTaskIdOffer
from inductiva.client.apis.paths.tasks_task_id_message import TasksTaskIdMessage
from inductiva.client.apis.paths.admin_users import AdminUsers
from inductiva.client.apis.paths.admin_users_email_terms_and_conditions import AdminUsersEmailTermsAndConditions
from inductiva.client.apis.paths.admin_users_username_organization import AdminUsersUsernameOrganization
from inductiva.client.apis.paths.admin_users_username_tier import AdminUsersUsernameTier
from inductiva.client.apis.paths.admin_users_username_credits import AdminUsersUsernameCredits
from inductiva.client.apis.paths.admin_users_email_api_key import AdminUsersEmailApiKey
from inductiva.client.apis.paths.admin_users_email import AdminUsersEmail
from inductiva.client.apis.paths.admin_users_username_storage_size_fs import AdminUsersUsernameStorageSizeFs
from inductiva.client.apis.paths.admin_users_username_storage_size import AdminUsersUsernameStorageSize
from inductiva.client.apis.paths.admin_users_username_tasks import AdminUsersUsernameTasks
from inductiva.client.apis.paths.admin_users_username_capabilities import AdminUsersUsernameCapabilities
from inductiva.client.apis.paths.admin_groups import AdminGroups
from inductiva.client.apis.paths.admin_groups_active import AdminGroupsActive
from inductiva.client.apis.paths.admin_active_tasks import AdminActiveTasks
from inductiva.client.apis.paths.admin_groups_machine_group_id_terminate import AdminGroupsMachineGroupIdTerminate
from inductiva.client.apis.paths.admin_organizations import AdminOrganizations
from inductiva.client.apis.paths.admin_organizations_organization_id import AdminOrganizationsOrganizationId
from inductiva.client.apis.paths.admin_organizations_costs import AdminOrganizationsCosts
from inductiva.client.apis.paths.admin_tiers import AdminTiers
from inductiva.client.apis.paths.admin_terminate_machine_groups_credits_exhausted import AdminTerminateMachineGroupsCreditsExhausted
from inductiva.client.apis.paths.task_runner_register import TaskRunnerRegister
from inductiva.client.apis.paths.task_runner_machine_id import TaskRunnerMachineId
from inductiva.client.apis.paths.task_runner_machine_id_task import TaskRunnerMachineIdTask
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_message import TaskRunnerMachineIdTaskTaskIdMessage
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_message_unblock import TaskRunnerMachineIdTaskTaskIdMessageUnblock
from inductiva.client.apis.paths.task_runner_machine_id_event import TaskRunnerMachineIdEvent
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_operation import TaskRunnerMachineIdTaskTaskIdOperation
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_operation_operation_id_done import TaskRunnerMachineIdTaskTaskIdOperationOperationIdDone
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_download_input_url import TaskRunnerMachineIdTaskTaskIdDownloadInputUrl
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_upload_output_url import TaskRunnerMachineIdTaskTaskIdUploadOutputUrl
from inductiva.client.apis.paths.task_runner_machine_id_task_task_id_metric import TaskRunnerMachineIdTaskTaskIdMetric
from inductiva.client.apis.paths.task_runner_machine_id_resize_disk import TaskRunnerMachineIdResizeDisk
from inductiva.client.apis.paths.task_runner_machine_id_resize_disk_done import TaskRunnerMachineIdResizeDiskDone
from inductiva.client.apis.paths.task_runner_machine_id_download_urls import TaskRunnerMachineIdDownloadUrls
from inductiva.client.apis.paths.compute_group import ComputeGroup
from inductiva.client.apis.paths.compute_group_start import ComputeGroupStart
from inductiva.client.apis.paths.compute_price import ComputePrice
from inductiva.client.apis.paths.compute_groups import ComputeGroups
from inductiva.client.apis.paths.compute_groups_history import ComputeGroupsHistory
from inductiva.client.apis.paths.compute_group_status import ComputeGroupStatus
from inductiva.client.apis.paths.compute_machine_types import ComputeMachineTypes
from inductiva.client.apis.paths.compute_group_name import ComputeGroupName
from inductiva.client.apis.paths.storage_size import StorageSize
from inductiva.client.apis.paths.storage_cost import StorageCost
from inductiva.client.apis.paths.storage_contents import StorageContents
from inductiva.client.apis.paths.storage_input_url import StorageInputUrl
from inductiva.client.apis.paths.storage_signed_urls import StorageSignedUrls
from inductiva.client.apis.paths.storage_input_notify import StorageInputNotify
from inductiva.client.apis.paths.storage_input_remote import StorageInputRemote
from inductiva.client.apis.paths.storage_ import Storage
from inductiva.client.apis.paths.storage_export import StorageExport
from inductiva.client.apis.paths.storage_operations_operation_id import StorageOperationsOperationId
from inductiva.client.apis.paths.storage_operations import StorageOperations
from inductiva.client.apis.paths.version import Version
from inductiva.client.apis.paths.version_check import VersionCheck
from inductiva.client.apis.paths.users_quotas import UsersQuotas
from inductiva.client.apis.paths.users_info import UsersInfo
from inductiva.client.apis.paths.users_capabilities import UsersCapabilities
from inductiva.client.apis.paths.users_costs import UsersCosts
from inductiva.client.apis.paths.users_organization_costs import UsersOrganizationCosts
from inductiva.client.apis.paths.projects import Projects
from inductiva.client.apis.paths.projects_name import ProjectsName
from inductiva.client.apis.paths.metrics_users_username_activity import MetricsUsersUsernameActivity
from inductiva.client.apis.paths.metrics_users_username_cost_over_time import MetricsUsersUsernameCostOverTime
from inductiva.client.apis.paths.metrics_users_username_task_status_overview import MetricsUsersUsernameTaskStatusOverview
from inductiva.client.apis.paths.metrics_users_username_computation_time_trend import MetricsUsersUsernameComputationTimeTrend
from inductiva.client.apis.paths.metrics_users_username_tasks_overview import MetricsUsersUsernameTasksOverview
from inductiva.client.apis.paths.metrics_users_username_most_used_machine_types import MetricsUsersUsernameMostUsedMachineTypes
from inductiva.client.apis.paths.metrics_users_username_most_used_simulators_overview import MetricsUsersUsernameMostUsedSimulatorsOverview
from inductiva.client.apis.paths.metrics_usage_statistics import MetricsUsageStatistics

PathToApi = typing_extensions.TypedDict(
    'PathToApi', {
        PathValues.TASKS_AUTH:
            TasksAuth,
        PathValues.TASKS_SUBMIT:
            TasksSubmit,
        PathValues.TASKS_TASK_ID_INPUT_UPLOAD_URL:
            TasksTaskIdInputUploadUrl,
        PathValues.TASKS_TASK_ID_INPUT_UPLOADED:
            TasksTaskIdInputUploaded,
        PathValues.TASKS_TASK_ID:
            TasksTaskId,
        PathValues.TASKS:
            Tasks,
        PathValues.TASKS_TASK_ID_STATUS:
            TasksTaskIdStatus,
        PathValues.TASKS_TASK_ID_OUTPUT_LIST:
            TasksTaskIdOutputList,
        PathValues.TASKS_TASK_ID_DOWNLOAD_INPUT_URL:
            TasksTaskIdDownloadInputUrl,
        PathValues.TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL:
            TasksTaskIdDownloadOutputUrl,
        PathValues.TASKS_TASK_ID_RESUBMIT:
            TasksTaskIdResubmit,
        PathValues.TASKS_TASK_ID_KILL:
            TasksTaskIdKill,
        PathValues.TASKS_TASK_ID_DISABLE_LOGS:
            TasksTaskIdDisableLogs,
        PathValues.TASKS_TASK_ID_FILES:
            TasksTaskIdFiles,
        PathValues.TASKS_TASK_ID_REGISTER:
            TasksTaskIdRegister,
        PathValues.TASKS_TASK_ID_OFFER:
            TasksTaskIdOffer,
        PathValues.TASKS_TASK_ID_MESSAGE:
            TasksTaskIdMessage,
        PathValues.ADMIN_USERS:
            AdminUsers,
        PathValues.ADMIN_USERS_EMAIL_TERMS_AND_CONDITIONS:
            AdminUsersEmailTermsAndConditions,
        PathValues.ADMIN_USERS_USERNAME_ORGANIZATION:
            AdminUsersUsernameOrganization,
        PathValues.ADMIN_USERS_USERNAME_TIER:
            AdminUsersUsernameTier,
        PathValues.ADMIN_USERS_USERNAME_CREDITS:
            AdminUsersUsernameCredits,
        PathValues.ADMIN_USERS_EMAIL_API_KEY:
            AdminUsersEmailApiKey,
        PathValues.ADMIN_USERS_EMAIL:
            AdminUsersEmail,
        PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE_FS:
            AdminUsersUsernameStorageSizeFs,
        PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE:
            AdminUsersUsernameStorageSize,
        PathValues.ADMIN_USERS_USERNAME_TASKS:
            AdminUsersUsernameTasks,
        PathValues.ADMIN_USERS_USERNAME_CAPABILITIES:
            AdminUsersUsernameCapabilities,
        PathValues.ADMIN_GROUPS:
            AdminGroups,
        PathValues.ADMIN_GROUPS_ACTIVE:
            AdminGroupsActive,
        PathValues.ADMIN_ACTIVE_TASKS:
            AdminActiveTasks,
        PathValues.ADMIN_GROUPS_MACHINE_GROUP_ID_TERMINATE:
            AdminGroupsMachineGroupIdTerminate,
        PathValues.ADMIN_ORGANIZATIONS:
            AdminOrganizations,
        PathValues.ADMIN_ORGANIZATIONS_ORGANIZATION_ID:
            AdminOrganizationsOrganizationId,
        PathValues.ADMIN_ORGANIZATIONS_COSTS:
            AdminOrganizationsCosts,
        PathValues.ADMIN_TIERS:
            AdminTiers,
        PathValues.ADMIN_TERMINATE_MACHINE_GROUPS_CREDITS_EXHAUSTED:
            AdminTerminateMachineGroupsCreditsExhausted,
        PathValues.TASKRUNNER_REGISTER:
            TaskRunnerRegister,
        PathValues.TASKRUNNER_MACHINE_ID:
            TaskRunnerMachineId,
        PathValues.TASKRUNNER_MACHINE_ID_TASK:
            TaskRunnerMachineIdTask,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_MESSAGE:
            TaskRunnerMachineIdTaskTaskIdMessage,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK:
            TaskRunnerMachineIdTaskTaskIdMessageUnblock,
        PathValues.TASKRUNNER_MACHINE_ID_EVENT:
            TaskRunnerMachineIdEvent,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_OPERATION:
            TaskRunnerMachineIdTaskTaskIdOperation,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_OPERATION_OPERATION_ID_DONE:
            TaskRunnerMachineIdTaskTaskIdOperationOperationIdDone,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL:
            TaskRunnerMachineIdTaskTaskIdDownloadInputUrl,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL:
            TaskRunnerMachineIdTaskTaskIdUploadOutputUrl,
        PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_METRIC:
            TaskRunnerMachineIdTaskTaskIdMetric,
        PathValues.TASKRUNNER_MACHINE_ID_RESIZE_DISK:
            TaskRunnerMachineIdResizeDisk,
        PathValues.TASKRUNNER_MACHINE_ID_RESIZE_DISK_DONE:
            TaskRunnerMachineIdResizeDiskDone,
        PathValues.TASKRUNNER_MACHINE_ID_DOWNLOAD_URLS:
            TaskRunnerMachineIdDownloadUrls,
        PathValues.COMPUTE_GROUP:
            ComputeGroup,
        PathValues.COMPUTE_GROUP_START:
            ComputeGroupStart,
        PathValues.COMPUTE_PRICE:
            ComputePrice,
        PathValues.COMPUTE_GROUPS:
            ComputeGroups,
        PathValues.COMPUTE_GROUPS_HISTORY:
            ComputeGroupsHistory,
        PathValues.COMPUTE_GROUP_STATUS:
            ComputeGroupStatus,
        PathValues.COMPUTE_MACHINE_TYPES:
            ComputeMachineTypes,
        PathValues.COMPUTE_GROUP_NAME:
            ComputeGroupName,
        PathValues.STORAGE_SIZE:
            StorageSize,
        PathValues.STORAGE_COST:
            StorageCost,
        PathValues.STORAGE_CONTENTS:
            StorageContents,
        PathValues.STORAGE_INPUT_URL:
            StorageInputUrl,
        PathValues.STORAGE_SIGNEDURLS:
            StorageSignedUrls,
        PathValues.STORAGE_INPUT_NOTIFY:
            StorageInputNotify,
        PathValues.STORAGE_INPUT_REMOTE:
            StorageInputRemote,
        PathValues.STORAGE_:
            Storage,
        PathValues.STORAGE_EXPORT:
            StorageExport,
        PathValues.STORAGE_OPERATIONS_OPERATION_ID:
            StorageOperationsOperationId,
        PathValues.STORAGE_OPERATIONS:
            StorageOperations,
        PathValues.VERSION:
            Version,
        PathValues.VERSIONCHECK:
            VersionCheck,
        PathValues.USERS_QUOTAS:
            UsersQuotas,
        PathValues.USERS_INFO:
            UsersInfo,
        PathValues.USERS_CAPABILITIES:
            UsersCapabilities,
        PathValues.USERS_COSTS:
            UsersCosts,
        PathValues.USERS_ORGANIZATION_COSTS:
            UsersOrganizationCosts,
        PathValues.PROJECTS:
            Projects,
        PathValues.PROJECTS_NAME:
            ProjectsName,
        PathValues.METRICS_USERS_USERNAME_ACTIVITY:
            MetricsUsersUsernameActivity,
        PathValues.METRICS_USERS_USERNAME_COST_OVER_TIME:
            MetricsUsersUsernameCostOverTime,
        PathValues.METRICS_USERS_USERNAME_TASK_STATUS_OVERVIEW:
            MetricsUsersUsernameTaskStatusOverview,
        PathValues.METRICS_USERS_USERNAME_COMPUTATION_TIME_TREND:
            MetricsUsersUsernameComputationTimeTrend,
        PathValues.METRICS_USERS_USERNAME_TASKS_OVERVIEW:
            MetricsUsersUsernameTasksOverview,
        PathValues.METRICS_USERS_USERNAME_MOST_USED_MACHINE_TYPES:
            MetricsUsersUsernameMostUsedMachineTypes,
        PathValues.METRICS_USERS_USERNAME_MOST_USED_SIMULATORS_OVERVIEW:
            MetricsUsersUsernameMostUsedSimulatorsOverview,
        PathValues.METRICS_USAGE_STATISTICS:
            MetricsUsageStatistics,
    })

path_to_api = PathToApi({
    PathValues.TASKS_AUTH:
        TasksAuth,
    PathValues.TASKS_SUBMIT:
        TasksSubmit,
    PathValues.TASKS_TASK_ID_INPUT_UPLOAD_URL:
        TasksTaskIdInputUploadUrl,
    PathValues.TASKS_TASK_ID_INPUT_UPLOADED:
        TasksTaskIdInputUploaded,
    PathValues.TASKS_TASK_ID:
        TasksTaskId,
    PathValues.TASKS:
        Tasks,
    PathValues.TASKS_TASK_ID_STATUS:
        TasksTaskIdStatus,
    PathValues.TASKS_TASK_ID_OUTPUT_LIST:
        TasksTaskIdOutputList,
    PathValues.TASKS_TASK_ID_DOWNLOAD_INPUT_URL:
        TasksTaskIdDownloadInputUrl,
    PathValues.TASKS_TASK_ID_DOWNLOAD_OUTPUT_URL:
        TasksTaskIdDownloadOutputUrl,
    PathValues.TASKS_TASK_ID_RESUBMIT:
        TasksTaskIdResubmit,
    PathValues.TASKS_TASK_ID_KILL:
        TasksTaskIdKill,
    PathValues.TASKS_TASK_ID_DISABLE_LOGS:
        TasksTaskIdDisableLogs,
    PathValues.TASKS_TASK_ID_FILES:
        TasksTaskIdFiles,
    PathValues.TASKS_TASK_ID_REGISTER:
        TasksTaskIdRegister,
    PathValues.TASKS_TASK_ID_OFFER:
        TasksTaskIdOffer,
    PathValues.TASKS_TASK_ID_MESSAGE:
        TasksTaskIdMessage,
    PathValues.ADMIN_USERS:
        AdminUsers,
    PathValues.ADMIN_USERS_EMAIL_TERMS_AND_CONDITIONS:
        AdminUsersEmailTermsAndConditions,
    PathValues.ADMIN_USERS_USERNAME_ORGANIZATION:
        AdminUsersUsernameOrganization,
    PathValues.ADMIN_USERS_USERNAME_TIER:
        AdminUsersUsernameTier,
    PathValues.ADMIN_USERS_USERNAME_CREDITS:
        AdminUsersUsernameCredits,
    PathValues.ADMIN_USERS_EMAIL_API_KEY:
        AdminUsersEmailApiKey,
    PathValues.ADMIN_USERS_EMAIL:
        AdminUsersEmail,
    PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE_FS:
        AdminUsersUsernameStorageSizeFs,
    PathValues.ADMIN_USERS_USERNAME_STORAGE_SIZE:
        AdminUsersUsernameStorageSize,
    PathValues.ADMIN_USERS_USERNAME_TASKS:
        AdminUsersUsernameTasks,
    PathValues.ADMIN_USERS_USERNAME_CAPABILITIES:
        AdminUsersUsernameCapabilities,
    PathValues.ADMIN_GROUPS:
        AdminGroups,
    PathValues.ADMIN_GROUPS_ACTIVE:
        AdminGroupsActive,
    PathValues.ADMIN_ACTIVE_TASKS:
        AdminActiveTasks,
    PathValues.ADMIN_GROUPS_MACHINE_GROUP_ID_TERMINATE:
        AdminGroupsMachineGroupIdTerminate,
    PathValues.ADMIN_ORGANIZATIONS:
        AdminOrganizations,
    PathValues.ADMIN_ORGANIZATIONS_ORGANIZATION_ID:
        AdminOrganizationsOrganizationId,
    PathValues.ADMIN_ORGANIZATIONS_COSTS:
        AdminOrganizationsCosts,
    PathValues.ADMIN_TIERS:
        AdminTiers,
    PathValues.ADMIN_TERMINATE_MACHINE_GROUPS_CREDITS_EXHAUSTED:
        AdminTerminateMachineGroupsCreditsExhausted,
    PathValues.TASKRUNNER_REGISTER:
        TaskRunnerRegister,
    PathValues.TASKRUNNER_MACHINE_ID:
        TaskRunnerMachineId,
    PathValues.TASKRUNNER_MACHINE_ID_TASK:
        TaskRunnerMachineIdTask,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_MESSAGE:
        TaskRunnerMachineIdTaskTaskIdMessage,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_MESSAGE_UNBLOCK:
        TaskRunnerMachineIdTaskTaskIdMessageUnblock,
    PathValues.TASKRUNNER_MACHINE_ID_EVENT:
        TaskRunnerMachineIdEvent,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_OPERATION:
        TaskRunnerMachineIdTaskTaskIdOperation,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_OPERATION_OPERATION_ID_DONE:
        TaskRunnerMachineIdTaskTaskIdOperationOperationIdDone,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_DOWNLOAD_INPUT_URL:
        TaskRunnerMachineIdTaskTaskIdDownloadInputUrl,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_UPLOAD_OUTPUT_URL:
        TaskRunnerMachineIdTaskTaskIdUploadOutputUrl,
    PathValues.TASKRUNNER_MACHINE_ID_TASK_TASK_ID_METRIC:
        TaskRunnerMachineIdTaskTaskIdMetric,
    PathValues.TASKRUNNER_MACHINE_ID_RESIZE_DISK:
        TaskRunnerMachineIdResizeDisk,
    PathValues.TASKRUNNER_MACHINE_ID_RESIZE_DISK_DONE:
        TaskRunnerMachineIdResizeDiskDone,
    PathValues.TASKRUNNER_MACHINE_ID_DOWNLOAD_URLS:
        TaskRunnerMachineIdDownloadUrls,
    PathValues.COMPUTE_GROUP:
        ComputeGroup,
    PathValues.COMPUTE_GROUP_START:
        ComputeGroupStart,
    PathValues.COMPUTE_PRICE:
        ComputePrice,
    PathValues.COMPUTE_GROUPS:
        ComputeGroups,
    PathValues.COMPUTE_GROUPS_HISTORY:
        ComputeGroupsHistory,
    PathValues.COMPUTE_GROUP_STATUS:
        ComputeGroupStatus,
    PathValues.COMPUTE_MACHINE_TYPES:
        ComputeMachineTypes,
    PathValues.COMPUTE_GROUP_NAME:
        ComputeGroupName,
    PathValues.STORAGE_SIZE:
        StorageSize,
    PathValues.STORAGE_COST:
        StorageCost,
    PathValues.STORAGE_CONTENTS:
        StorageContents,
    PathValues.STORAGE_INPUT_URL:
        StorageInputUrl,
    PathValues.STORAGE_SIGNEDURLS:
        StorageSignedUrls,
    PathValues.STORAGE_INPUT_NOTIFY:
        StorageInputNotify,
    PathValues.STORAGE_INPUT_REMOTE:
        StorageInputRemote,
    PathValues.STORAGE_:
        Storage,
    PathValues.STORAGE_EXPORT:
        StorageExport,
    PathValues.STORAGE_OPERATIONS_OPERATION_ID:
        StorageOperationsOperationId,
    PathValues.STORAGE_OPERATIONS:
        StorageOperations,
    PathValues.VERSION:
        Version,
    PathValues.VERSIONCHECK:
        VersionCheck,
    PathValues.USERS_QUOTAS:
        UsersQuotas,
    PathValues.USERS_INFO:
        UsersInfo,
    PathValues.USERS_CAPABILITIES:
        UsersCapabilities,
    PathValues.USERS_COSTS:
        UsersCosts,
    PathValues.USERS_ORGANIZATION_COSTS:
        UsersOrganizationCosts,
    PathValues.PROJECTS:
        Projects,
    PathValues.PROJECTS_NAME:
        ProjectsName,
    PathValues.METRICS_USERS_USERNAME_ACTIVITY:
        MetricsUsersUsernameActivity,
    PathValues.METRICS_USERS_USERNAME_COST_OVER_TIME:
        MetricsUsersUsernameCostOverTime,
    PathValues.METRICS_USERS_USERNAME_TASK_STATUS_OVERVIEW:
        MetricsUsersUsernameTaskStatusOverview,
    PathValues.METRICS_USERS_USERNAME_COMPUTATION_TIME_TREND:
        MetricsUsersUsernameComputationTimeTrend,
    PathValues.METRICS_USERS_USERNAME_TASKS_OVERVIEW:
        MetricsUsersUsernameTasksOverview,
    PathValues.METRICS_USERS_USERNAME_MOST_USED_MACHINE_TYPES:
        MetricsUsersUsernameMostUsedMachineTypes,
    PathValues.METRICS_USERS_USERNAME_MOST_USED_SIMULATORS_OVERVIEW:
        MetricsUsersUsernameMostUsedSimulatorsOverview,
    PathValues.METRICS_USAGE_STATISTICS:
        MetricsUsageStatistics,
})



"""Command to list directories in a task."""
from typing import TextIO
import argparse
import sys
import asyncio

from inductiva import _cli, tasks


def list_files(args: argparse.Namespace, fout: TextIO = sys.stdout):
    task_id = args.id
    task = tasks.Task(task_id)
    directories = asyncio.run(task._list_files())  # pylint: disable=protected-access
    print(directories, file=fout)
    return 0


def register(parser):
    """Register the info tasks command."""
    subparser = parser.add_parser(
        "list-files",
        help="Lists files generated by a task recursively.",
        formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = ("The `inductiva tasks list-files` command lists "
                             "the contents of a task's working directory while "
                             "the task in progress.")

    _cli.utils.add_watch_argument(subparser)
    subparser.add_argument("id", type=str, help="ID of the task to list files.")
    subparser.set_defaults(func=list_files)



"""Launches a Task-Runner via CLI."""
from typing import TextIO
import argparse
import threading
import sys
import os
from inductiva import _cli, constants, _api_key, api_url

_docker_imported = True
try:
    import docker
    from docker.errors import DockerException
except ImportError:
    _docker_imported = False


def generator_thread(container, fout: TextIO):
    """Runs a generator in a separate thread."""
    for line in container.logs(stream=True):
        print(line.decode("utf-8"), end="", file=fout)


def join_container_streams(*containers, fout: TextIO = sys.stdout):
    """Join multiple containers into one."""
    container_threads = []

    for container in containers:
        thread = threading.Thread(target=generator_thread,
                                  args=(container, fout))
        thread.start()
        container_threads.append(thread)

    for thread in container_threads:
        thread.join()


def launch_task_runner(args, fout: TextIO = sys.stdout):
    """Launches a Task-Runner."""
    if not _docker_imported:
        print(
            "Docker Python API not installed, please run "
            "'pip install inductiva[task-runner]' to install it",
            file=fout)
        return

    try:
        client = docker.from_env()
    except DockerException as e:
        print(f"Failed to connect to Docker: {e}", file=fout)
        print(
            "Please make sure Docker is running "
            "and you have the necessary permissions.",
            file=fout)
        return

    task_runner = client.containers.list(filters={"name": "task-runner"})
    file_tracker = client.containers.list(filters={"name": "file-tracker"})

    if task_runner or file_tracker:
        print(
            "Task-Runner already running. "
            "Please stop it before launching a new one.",
            file=fout)
        return

    client.images.pull(constants.FILE_TRACKER_IMAGE, platform="linux/amd64")
    file_tracker_container = client.containers.run(
        image=constants.FILE_TRACKER_IMAGE,
        name="file-tracker",
        environment={
            "API_URL": api_url,
            "USER_API_KEY": _api_key.get(),
        },
        volumes={
            "workdir": {
                "bind": "/workdir",
                "mode": "rw"
            },
        },
        network="host",
        platform="linux/amd64",
        detach=True,
        auto_remove=True,
    )

    client.images.pull(constants.TASK_RUNNER_IMAGE, platform="linux/amd64")

    apptainer_path = "apptainer"
    os.makedirs(apptainer_path, exist_ok=True)
    os.chmod(apptainer_path, 0o777)
    apptainer_full_path = os.path.abspath(apptainer_path)

    task_runner_container = client.containers.run(
        image=constants.TASK_RUNNER_IMAGE,
        name="task-runner",
        environment={
            "USER_API_KEY": _api_key.get(),
            "API_URL": api_url,
            "MACHINE_GROUP_NAME": args.machine_group_name,
            "HOST_NAME": args.hostname,
        },
        mounts=[
            docker.types.Mount(target="/executer-images",
                               source=apptainer_full_path,
                               type="bind")
        ],
        volumes={
            "workdir": {
                "bind": "/workdir",
                "mode": "rw"
            },
        },
        network="host",
        privileged=True,
        platform="linux/amd64",
        detach=True,
        auto_remove=True,
    )

    print(
        "File-Tracker launched "
        f"with container ID: {file_tracker_container.short_id}",
        file=fout)
    print(
        "Task-Runner launched "
        f"with container ID: {task_runner_container.short_id}",
        file=fout)

    if not args.detach:
        try:
            join_container_streams(task_runner_container,
                                   file_tracker_container)
        except KeyboardInterrupt:
            print("Interrupted. Stopping containers...", file=fout)
            file_tracker_container.stop()
            task_runner_container.stop()


def register(parser):
    """Register the launch task-runner command."""
    subparser = parser.add_parser("launch",
                                  help="Launches a Task-Runner.",
                                  formatter_class=argparse.RawTextHelpFormatter)

    subparser.description = (
        "The `inductiva task-runner launch` command provides "
        "a way to launch a Task-Runner on the platform.")

    _cli.utils.add_watch_argument(subparser)
    subparser.add_argument(
        "machine_group_name",
        type=str,
        help="Name of the machine group to launch the Task-Runner.")

    subparser.add_argument("--hostname",
                           "-ho",
                           type=str,
                           default=os.uname().nodename,
                           help="Hostname of the Task-Runner.")

    subparser.add_argument("--detach",
                           "-d",
                           action="store_true",
                           help="Run the task-runner in the background.")

    subparser.set_defaults(func=launch_task_runner)



# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class OrganizationUsers(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "identity_provider_id",
            "name",
            "creation_ts",
            "id",
            "total_users",
            "users",
            "status",
        }

        class properties:
            id = schemas.StrSchema
            identity_provider_id = schemas.StrSchema
            name = schemas.StrSchema

            @staticmethod
            def status() -> typing.Type['OrgStatus']:
                return OrgStatus

            creation_ts = schemas.DateTimeSchema
            total_users = schemas.IntSchema

            class users(schemas.ListSchema):

                class MetaOapg:
                    items = schemas.StrSchema

                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple[typing.Union[
                        MetaOapg.items,
                        str,
                    ]], typing.List[typing.Union[
                        MetaOapg.items,
                        str,
                    ]]],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                ) -> 'users':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )

                def __getitem__(self, i: int) -> MetaOapg.items:
                    return super().__getitem__(i)

            __annotations__ = {
                "id": id,
                "identity_provider_id": identity_provider_id,
                "name": name,
                "status": status,
                "creation_ts": creation_ts,
                "total_users": total_users,
                "users": users,
            }

    identity_provider_id: MetaOapg.properties.identity_provider_id
    name: MetaOapg.properties.name
    creation_ts: MetaOapg.properties.creation_ts
    id: MetaOapg.properties.id
    total_users: MetaOapg.properties.total_users
    users: MetaOapg.properties.users
    status: 'OrgStatus'

    @typing.overload
    def __getitem__(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["identity_provider_id"]
    ) -> MetaOapg.properties.identity_provider_id:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def __getitem__(self,
                    name: typing_extensions.Literal["status"]) -> 'OrgStatus':
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["creation_ts"]
    ) -> MetaOapg.properties.creation_ts:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["total_users"]
    ) -> MetaOapg.properties.total_users:
        ...

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["users"]
    ) -> MetaOapg.properties.users:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "identity_provider_id",
        "name",
        "status",
        "creation_ts",
        "total_users",
        "users",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
            self,
            name: typing_extensions.Literal["id"]) -> MetaOapg.properties.id:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["identity_provider_id"]
    ) -> MetaOapg.properties.identity_provider_id:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["name"]
    ) -> MetaOapg.properties.name:
        ...

    @typing.overload
    def get_item_oapg(self,
                      name: typing_extensions.Literal["status"]) -> 'OrgStatus':
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["creation_ts"]
    ) -> MetaOapg.properties.creation_ts:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["total_users"]
    ) -> MetaOapg.properties.total_users:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["users"]
    ) -> MetaOapg.properties.users:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "id",
        "identity_provider_id",
        "name",
        "status",
        "creation_ts",
        "total_users",
        "users",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        identity_provider_id: typing.Union[
            MetaOapg.properties.identity_provider_id,
            str,
        ],
        name: typing.Union[
            MetaOapg.properties.name,
            str,
        ],
        creation_ts: typing.Union[
            MetaOapg.properties.creation_ts,
            str,
            datetime,
        ],
        id: typing.Union[
            MetaOapg.properties.id,
            str,
        ],
        total_users: typing.Union[
            MetaOapg.properties.total_users,
            decimal.Decimal,
            int,
        ],
        users: typing.Union[
            MetaOapg.properties.users,
            list,
            tuple,
        ],
        status: 'OrgStatus',
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'OrganizationUsers':
        return super().__new__(
            cls,
            *_args,
            identity_provider_id=identity_provider_id,
            name=name,
            creation_ts=creation_ts,
            id=id,
            total_users=total_users,
            users=users,
            status=status,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.org_status import OrgStatus
